package com.artificialsolutions.asolfbconnector.webapp.listeners;

import java.io.File;
import java.io.IOException;
import java.util.Properties;

import jakarta.servlet.ServletContextEvent;
import jakarta.servlet.ServletContextListener;

import com.artificialsolutions.asolfbconnector.common.Config;
import com.artificialsolutions.common.Utils;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * An implementation of {@link jakarta.servlet.ServletContextListener} used in this application.
 */
public class ServletContextListenerImpl implements ServletContextListener {

    /**
     * Receives notification that the web application initialization process is starting. It reads the
     * init.properties file which contains the "configFile" property pointing to the global
     * configuration file.
     *
     * @param evt {@inheritDoc}
     */
    @Override
    public void contextInitialized(final ServletContextEvent evt) {
        final Logger logger = LoggerFactory.getLogger(getClass());
        logger.debug("Initializing servlet context");
        File file = Utils.getInternalFile(getClass().getClassLoader(), "init.properties");
        Properties props = null;
        try {
            props = Config.getPropertiesFromFile(file);
            logger.info("Init file [{}] is read", file);
        } catch (final IOException ex) {
            logger.error("Failure reading init file [{}]", file, ex);
            return;
        }
        String s = props.getProperty("configFile");
        props = null;
        if (s == null || s.isEmpty()) {
            logger.error("Empty or absent property [configFile] in [{}]", file);
            return;
        }
        file = new File(s);
        s = null;
        try {
            evt.getServletContext().setAttribute("config", new Config(file));
            logger.info("Configuration is read from [{}]", file);
        } catch (final Exception ex) {
            logger.error("Failure reading configuration from [{}]", file, ex);
        }
    }

    /**
     * Receives notification that the ServletContext is about to be shut down.
     *
     * @param evt {@inheritDoc}
     */
    @Override
    public void contextDestroyed(final ServletContextEvent evt) {
        final Logger logger = LoggerFactory.getLogger(getClass());
        logger.debug("Destroying servlet context");
        evt.getServletContext().removeAttribute("config");
    }
}
