package com.artificialsolutions.asolfbconnector.teneoengine;

import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;
import java.util.function.BiConsumer;

import org.junit.jupiter.api.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.artificialsolutions.asolfbconnector.common.Config;
import com.artificialsolutions.asolfbconnector.teneoengine.TeneoEngineClient;
import com.artificialsolutions.common.Utils;

import jakarta.json.JsonObject;

@DisplayName("teneoEngineClient")
public class TestTeneoEngineClient {

    static final Config config;
    static final int teneoThreadWait;

    private static RuntimeException failure(final String msg, final Throwable thr) {
        System.err.println(msg);
        return thr == null ? new RuntimeException(msg) : new RuntimeException(msg, thr);
    }

    static {
        File file = Utils.getInternalFile(TestTeneoEngineClient.class.getClassLoader(), "init.properties");
        Properties props = null;
        try {
            props = Config.getPropertiesFromFile(file);
        } catch (final IOException ex) {
            throw failure("Failure reading from [" + file.getAbsolutePath() + ']', ex);
        }
        String sFile = props.getProperty("configFile");
        props = null;
        if (sFile == null || sFile.isEmpty()) {
            throw failure("No [configFile] in [" + file.getAbsolutePath() + ']', null);
        }
        file = new File(sFile);
        sFile = null;
        try {
            config = new Config(file);
        } catch (final Exception ex) {
            throw failure("Failure reading from [" + file.getAbsolutePath() + ']', ex);
        }
        teneoThreadWait = 2 * (config.getTeneoEngineConnectTimeoutMillis() + config.getTeneoEngineResponseTimeoutMillis());
    }

    private final Logger logger = LoggerFactory.getLogger(TestTeneoEngineClient.class);

    private final Object teneoRequestLock = new Object();

    private volatile boolean teneoRequestCompleted;
    private volatile String sessionId;

    @BeforeAll
    static void initAll() {
    }

    @BeforeEach
    void init() {
        synchronized (teneoRequestLock) {
            teneoRequestCompleted = false;
            sessionId = null;
        }
    }

    @Test
    @Disabled("Tempoparily disabled")
    void teneoEngineTest() throws Exception {
        logger.info("BEGIN teneoEngineTest");

        final TeneoEngineClient tec = new TeneoEngineClient(config);
        final Map<String, String> m = new HashMap<String, String>(2);
        m.put("viewtype", "tieapi");
        m.put("userinput", "hi");

        final BiConsumer<JsonObject, Throwable> responseHandler = (final JsonObject json, final Throwable thr) -> {
            final String sessionId;
            if (thr == null) {
                logger.trace("teneoEngineTest -- JSON {}", json);
                sessionId = json.getString("sessionId");
            } else {
                logger.warn("teneoEngineTest -- Throwable", thr);
                sessionId = null;
            }
            synchronized (teneoRequestLock) {
                teneoRequestCompleted = true;
                if (thr == null) {
                    if (this.sessionId == null) {
                        this.sessionId = sessionId;
                        logger.debug("teneoEngineTest -- Session ID assigned [{}]", sessionId);
                    } else if (this.sessionId.equals(sessionId)) {
                        logger.debug("teneoEngineTest -- Session sustained [{}]", sessionId);
                    } else {
                        logger.warn("teneoEngineTest -- Session lost from [{}] to [{}]", this.sessionId);
                    }
                }
                teneoRequestLock.notify();
            }
        };

        for (int i = 0; i < 10; i++) {
            synchronized (teneoRequestLock) {
                teneoRequestCompleted = false;
                logger.trace("teneoEngineTest before sendAsync");
                tec.sendAsync(m).whenComplete(responseHandler);
                logger.trace("teneoEngineTest after sendAsync");
                final long startMillis = System.currentTimeMillis();
                while (!teneoRequestCompleted && System.currentTimeMillis() - startMillis < teneoThreadWait) {
                    teneoRequestLock.wait(teneoThreadWait);
                }
            }
        }

        logger.info("END teneoEngineTest");
    }

    @AfterEach
    void tearDown() {
        synchronized (teneoRequestLock) {
            teneoRequestCompleted = false;
            sessionId = null;
        }
    }

    @AfterAll
    static void tearDownAll() {
    }
}
